@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,13 +36,18 @@ class VersionsControllerTest < Redmine::ControllerTest
     assert_response :success
 
     # Version with no date set appears
-    assert_select 'h3', :text => Version.find(3).name
+    assert_select 'h3', :text => "#{Version.find(3).name}"
+    assert_select 'span[class=?]', 'badge badge-status-open', :text => 'open'
+
     # Completed version doesn't appear
     assert_select 'h3', :text => Version.find(1).name, :count => 0
 
     # Context menu on issues
     assert_select "form[data-cm-url=?]", '/issues/context_menu'
     assert_select "div#sidebar" do
+      # Tracker checkboxes
+      assert_select 'input[type=hidden][name=?]', 'tracker_ids[]'
+      assert_select 'input[type=checkbox][name=?]', 'tracker_ids[]', 3
       # Links to versions anchors
       assert_select 'a[href=?]', '#2.0'
       # Links to completed versions in the sidebar
@@ -70,14 +77,17 @@ class VersionsControllerTest < Redmine::ControllerTest
   end
 
   def test_index_showing_subprojects_versions
-    @subproject_version = Version.create!(:project => Project.find(3), :name => "Subproject version")
+    version_name = "Subproject version"
+    Version.create!(:project => Project.find(3), :name => version_name)
     get :index, :params => {:project_id => 1, :with_subprojects => 1}
     assert_response :success
 
     # Shared version
     assert_select 'h3', :text => Version.find(4).name
     # Subproject version
-    assert_select 'h3', :text => /Subproject version/
+    assert_select 'h3', :text => /#{version_name}/
+    # Subproject checkbox
+    assert_select '#sidebar input[id=?][value=?]', "with_subprojects", 1
   end
 
   def test_index_should_prepend_shared_versions
@@ -94,11 +104,84 @@ class VersionsControllerTest < Redmine::ControllerTest
     end
   end
 
-  def test_show
-    get :show, :params => {:id => 2}
+  def test_index_should_show_issue_assignee
+    with_settings :gravatar_enabled => '1' do
+      Issue.generate!(:project_id => 3, :fixed_version_id => 4, :assigned_to => User.find_by_login('jsmith'))
+      Issue.generate!(:project_id => 3, :fixed_version_id => 4)
+
+      get :index, :params => {:project_id => 3}
+      assert_response :success
+
+      assert_select 'table.related-issues' do
+        assert_select 'tr.issue', :count => 2 do
+          assert_select 'img.gravatar[title=?]', 'Assignee: John Smith', :count => 1
+        end
+      end
+    end
+  end
+
+  def test_index_subproject_checkbox_should_check_descendants_visibility
+    project = Project.find(6)
+    project.is_public = false
+    project.save
+
+    @request.session[:user_id] = 2
+
+    get :index, :params => {:project_id => 5, :with_subprojects => 1}
     assert_response :success
 
-    assert_select 'h2', :text => /1.0/
+    # Subproject checkbox should not be shown
+    assert_select '#sidebar input[id=?]', "with_subprojects", :count => 0
+  end
+
+  def test_show
+    with_settings :gravatar_enabled => '0' do
+      get :show, :params => {:id => 2}
+      assert_response :success
+
+      assert_select 'h2', :text => /1.0/
+      assert_select 'span[class=?]', 'badge badge-status-locked', :text => 'locked'
+
+      # no issue avatar when gravatar is disabled
+      assert_select 'img.gravatar', :count => 0
+    end
+  end
+
+  def test_show_should_show_issue_assignee
+    with_settings :gravatar_enabled => '1' do
+      get :show, :params => {:id => 2}
+      assert_response :success
+
+      assert_select 'table.related-issues' do
+        assert_select 'tr.issue td.assigned_to', :count => 2 do
+          assert_select 'img.gravatar[title=?]', 'Assignee: Dave Lopper', :count => 1
+        end
+      end
+    end
+  end
+
+  def test_show_issue_calculations_should_take_into_account_only_visible_issues
+    issue_9 = Issue.find(9)
+    issue_9.fixed_version_id = 4
+    issue_9.estimated_hours = 3
+    issue_9.save!
+
+    issue_13 = Issue.find(13)
+    issue_13.fixed_version_id = 4
+    issue_13.estimated_hours = 2
+    issue_13.save!
+
+    @request.session[:user_id] = 7
+
+    get :show, :params => {:id => 4}
+    assert_response :success
+
+    assert_select 'p.progress-info' do
+      assert_select 'a', :text => '1 issue'
+      assert_select 'a', :text => '1 open'
+    end
+
+    assert_select '.time-tracking td.total-hours a:first-child', :text => '2:00 hours'
   end
 
   def test_show_should_link_to_spent_time_on_version
@@ -109,7 +192,7 @@ class VersionsControllerTest < Redmine::ControllerTest
     get :show, :params => {:id => version.id}
     assert_response :success
 
-    assert_select '.total-hours', :text => '7.20 hours'
+    assert_select '.total-hours', :text => '7:12 hours'
     assert_select '.total-hours a[href=?]', "/projects/ecookbook/time_entries?issue.fixed_version_id=#{version.id}&set_filter=1"
   end
 
@@ -126,6 +209,33 @@ class VersionsControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_show_should_round_down_progress_percentages
+    issue = Issue.find(12)
+    issue.estimated_hours = 40
+    issue.save!
+
+    with_settings :default_language => 'en' do
+      get :show, :params => {:id => 2}
+      assert_response :success
+
+      assert_select 'div.version-overview' do
+        assert_select 'table.progress-98' do
+          assert_select 'td[class=closed][title=?]', 'closed: 98%'
+          assert_select 'td[class=done][title=?]', '% Done: 99%'
+        end
+        assert_select 'p[class=percent]', :text => '99%'
+      end
+    end
+  end
+
+  def test_show_should_display_link_to_new_issue
+    @request.session[:user_id] = 1
+    get :show, :params => {:id => 3}
+
+    assert_response :success
+    assert_select 'a.icon.icon-add', :text => 'New issue'
+  end
+
   def test_new
     @request.session[:user_id] = 2
     get :new, :params => {:project_id => '1'}
@@ -138,7 +248,7 @@ class VersionsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 2
     get :new, :params => {:project_id => '1'}, :xhr => true
     assert_response :success
-    assert_equal 'text/javascript', response.content_type
+    assert_equal 'text/javascript', response.media_type
   end
 
   def test_create
@@ -162,7 +272,7 @@ class VersionsControllerTest < Redmine::ControllerTest
     assert_equal 1, version.project_id
 
     assert_response :success
-    assert_equal 'text/javascript', response.content_type
+    assert_equal 'text/javascript', response.media_type
     assert_include 'test_add_version_from_issue_form', response.body
   end
 
@@ -172,7 +282,7 @@ class VersionsControllerTest < Redmine::ControllerTest
       post :create, :params => {:project_id => '1', :version => {:name => ''}}, :xhr => true
     end
     assert_response :success
-    assert_equal 'text/javascript', response.content_type
+    assert_equal 'text/javascript', response.media_type
   end
 
   def test_get_edit
